#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Upgrade CanDo to generation 10.

CanDo demographics should be merged into SchoolTool demographics.
"""
from zope.app.generations.utility import findObjectsProviding
from zope.app.publication.zopepublication import ZopePublication
from zope.component.hooks import getSite, setSite
from zope.component import getAdapter

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.basicperson.interfaces import IDemographics
from schooltool.app.interfaces import IPluginInit

from cando.virginia.interfaces import IStudentDemographics


DEMOGRAPHICS_FIELDS_KEY = 'schooltool.basicperson.demographics_fields'


def evolve(context):
    root = context.connection.root().get(ZopePublication.root_name, None)

    apps = findObjectsProviding(root, ISchoolToolApplication)
    for app in apps:

        if DEMOGRAPHICS_FIELDS_KEY not in app:
            plugin = getAdapter(app, IPluginInit,
                                name="schooltool.basicperson.demographics")
            plugin()

        # Set custom CanDo demographics

        plugin = getAdapter(app, IPluginInit, name="cando.demographics")
        plugin()

        # Look for persons and fill their new SchoolTool demographics
        # with their old CanDo demographics

        persons = app['persons']
        old_site = getSite()
        setSite(app)
        for person in persons.values():
            student_demographics = IStudentDemographics(person)
            person_demographics = IDemographics(person)
            for field in ['state_report_id', 'ethnicity',
                          'grade_class']:
                value = getattr(student_demographics, field, None)
                if value:
                    person_demographics[field] = value
        setSite(old_site)
