#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""Competency Interfaces

$Id$
"""
__docformat__='restructuredtext'


import zope.schema
from zope.interface import Interface

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('cando')


class IStatistics(Interface):
    """Statistics for the gradebook"""

    values = zope.schema.Tuple(
        title = _(u"Values"),
        description = _(u"Values for statistics."))

    average = zope.schema.Float(
        title = _(u"Average"),
        description = _(u"The average of the competency."),
        required=False)

    median = zope.schema.Float(
        title = _(u"Median"),
        description=_(u"The median of the competency"),
        required=False)

    standardDeviation = zope.schema.Float(
        title = _(u"Standard Deviation"),
        description=_(u"The standard deviation of the competency."),
        required=False)

    variance = zope.schema.Float(
        title = _(u"Variance"),
        description=_(u"The variance of the competency."),
        required=False)


class IHaveGradebookInstructors(Interface):
    """Marker interface for gradebooks to have instructors."""


class IHaveCompetencyGradebook(Interface):
    """Marker interface for things that have gradebooks.

    This is used to have all the different stuff that support
    gradebook have the same navigation to the gradebook and
    the same url as well.
    """


class ICompetencyGradebook(IHaveGradebookInstructors):
    """The gradebook of a section.

    The gradebook provides an API that allows the user to treat it like a
    gradebook spreadsheet/table.
    """

    title = zope.schema.TextLine(
        title=u"Title",
        description=u"Title of the gradebook.",
        required=True)

    comprefDict = zope.schema.Dict(
        title=u'Competency Reference Dict',
        description=u'Competency reference dictionary in this gradebook.',
        required=True)

    students = zope.schema.Iterable(
        title=u'Students',
        description=u'Students in this gradebook.')

    def getEvaluation(student, globalid, default=None):
        """Get the evaluation of a student for a given competency."""

    def evaluate(student, globalid, score, evaluator=None):
        """Evaluate a student for a competency"""

    def resetCurrentCompGroup(person):
        """Resets the current competency group to the first group
        alphabetically in the person's list."""

    def getCurrentCompGroup(person):
        """Returns the current competency group of the person."""

    def setCurrentCompGroup(person, compgroup):
        """Sets the current competency group of the person."""

class IHaveScoreReports(Interface):
    """Marker interface for gradebooks that have score reports."""


class IHaveStudentScoreReports(Interface):
    """Marker interface for gradebooks that have score reports."""


class IGradebookUtilities(Interface):
    """Set of utilities for a gradebook."""

    def getCompetencyByKey(key, default=None):
        """Given the hashed key value of a competency, return the competency."""


class IStudentCompetencyGradebook(IHaveGradebookInstructors):
    """Glues together a specific student and a specific gradebook."""

    student = zope.schema.Object(
        title=u'Student',
        schema=Interface,
        required=False) # to be a generic entity

    title = zope.schema.TextLine(
        title=u"Title",
        description=u"Title of the gradebook.",
        required=True)

    comprefDict = zope.schema.Dict(
        title=u'Competency Reference Dict',
        description=u'Competency reference dictionary in this gradebook.')

    def getEvaluation(globalid, default=None):
        """Get the evaluation of a student for a given competency."""

    def evaluate(globalid, score, evaluator=None):
        """Evaluate a student for a competency"""

    def resetCurrentCompGroup(person):
        """Resets the current competency group to the first group
        alphabetically in the person's list."""

    def getCurrentCompGroup(person):
        """Returns the current competency group of the person."""

    def setCurrentCompGroup(person, compgroup):
        """Sets the current competency group of the person."""


class IGradebookInstructors(Interface):
    """Represents instructors in a gradebook."""

    instructors = zope.schema.List(
        title=u'Instructors',
        description=u'Instructors for a gradebook.',
        default=[])

    ## XXX: we should probably put a constraint on this.
