#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
from zope import schema
from zope.interface import Interface, Attribute
from zope.schema import Text, TextLine, Datetime, Object
from zope.container.interfaces import IContained, IContainer
from zope.container.constraints import containers, contains
from schooltool.person.interfaces import IPerson
from schooltool.course.interfaces import ISection
from schooltool.group.interfaces import IBaseGroup

from zope.i18nmessageid import MessageFactory

_ = MessageFactory('cando')

class IMessage(IContainer):
    """A message"""

    title = TextLine(
      title= _(u"Message Title"),
      description= _(u"Message Title"),
      required=True)

    body = Text(
      title= _(u"Message Body"),
      description= _(u"Message Body"),
      required=True)

    author = Object(
      schema=IPerson,
      title= _(u"Author"),
      description= _(u"Message Author"),
      required=False)

    created = Datetime(
      title= _(u"Time created"),
      description= _(u"Time created"),
      required=True)

    modified = Datetime(
      title= _(u"Last modified"),
      description= _(u"Last modified"),
      required=False)

    contains('cando.message.interfaces.IMessage')


class IMessageBox(IBaseGroup, IContainer):
    """A MessageBox containing Messages"""

    title = TextLine(
      title= _(u"MessageBox Title"),
      description= _(u"MessageBox Title"),
      required=False)

    section = Attribute("""Section of the MessageBox""")

    person = Attribute("""Person who owns this MessageBox""")

    managers = Attribute("""Managers of a MessageBox""")

    contains(IMessage)


class IForum(IMessageBox):
    """A MessageBox that is for a section"""


class ISectionMessageBox(IMessageBox):
    """A MessageBox that is for a student in a section"""

class IMessageContained(IContained):
    """Contained version of Message."""
    containers(IMessageBox, IMessage)


class IMessageBoxContainer(IContainer):
    """A MessageBox Container"""

    def add(messagebox):
        """Add a MessageBox with a (person, section) key."""
    def getMessageBox(person, section):
        """Get a MessageBox for the (person, section) key."""
    def remove(person, section):
        """Remove the reference for (person, section) key."""

    contains(IMessageBox)


class IMessageBoxContained(IContained):
    """Contained MessageBox."""
    containers(IMessageBoxContainer)


class IHaveMessageBox(Interface):
    """Marker interfaces for objects that have MessageBoxes."""


class IQueueItem(Interface):
    """A queue item"""

    context = schema.Object(
        schema=IMessage,
        title=u"Message",
        description=u"Queued Message",
        required=True)

    section = schema.TextLine(
        title=u"Section",
        description=u"Identifier of the associated Section",
        required=False)

    person = schema.TextLine(
        title=u"Person",
        description=u"Identifier of the associated Person",
        required=False)

    history = schema.List(
        title=u"History",
        description=u"Queue Item History",
        value_type=schema.List(title=u"Status"),
        required=False)

    status = Attribute("""Status of a queue item""")


class IQueue(Interface):
    """A queue"""

    context = schema.Object(
        schema=ISection,
        title=u"Section",
        description=u"Section",
        required=True)

    items = schema.List(
        title=u"Items",
        description=u"Queued items in the queue",
        required=False)

class IQueueable(Interface):
    """Marker interface for objects that are queuable, namely Messages"""


class IHaveQueue(Interface):
    """Marker interface for objects that have a queue, namely Sections"""

