#
# CanDo - Web based competency tracking system.
# Copyright (C) 2007 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
from zope.viewlet.interfaces import IViewletManager
from zope.viewlet.viewlet import CSSViewlet, JavaScriptViewlet
from zope.interface import Interface, implements
from zope.component import subscribers
from zope.contentprovider.interfaces import IContentProvider
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from z3c.menu.simple import menu
from zope.browsermenu.menu import getMenu

from schooltool.skin.skin import IActionMenuContext
from schooltool.skin.skin import IActionMenuManager, OrderedViewletManager
from schooltool.help.browser import HelpLink as SchoolToolHelpLink
from cando.message.interfaces import IMessageBox
from cando.skilldriver.interfaces import ISkillDriver, ISectionSkillDrivers
from cando.virginia.interfaces import ICompetencies
from cando.newskin import PersonDistinguisher


class ICSS(IViewletManager):
    """CSS viewlet manager."""


class IJavaScript(IViewletManager):
    """JavaScript viewlet manager."""


class IJQueryPlugins(IViewletManager):
    """JQuery Plugins viewlet manager."""


class IMenuBarMenuManager(IViewletManager):
    """Provides a viewlet hook for the entire menu bar."""


CandoCSSViewlet = CSSViewlet('cando.css')
CandoJSViewlet = JavaScriptViewlet('cando.js')

CompetencyCSSViewlet = CSSViewlet('competencies.css')
CompetencyJSViewlet = JavaScriptViewlet('competencies.js')

JQueryDimensionsPluginViewlet = JavaScriptViewlet('jquery.dimensions.js')
JQueryTooltipPluginViewlet = JavaScriptViewlet('jquery.tooltip.js')
TooltipCSSViewlet = CSSViewlet('tooltip.css')


class ISubMenuItem(Interface):
    """Marker interface for objects that will be displayed as items in submenus.
    """


class MenuBarMenuViewletManager(OrderedViewletManager):
    """Viewlet manager for displaying the various menus at the top of a page."""

    implements(IMenuBarMenuManager)
    
    def render(self):
        viewlets = [viewlet for viewlet in self.viewlets
            if viewlet.isAvailable()]
        return u'\n'.join([viewlet.render() for viewlet in viewlets])


class MenuBarMenuItem(menu.SimpleMenuItem, PersonDistinguisher):
    """Menu item viewlet MenuBarMenuViewletManager."""
    
    template = ViewPageTemplateFile('templates/menuBarViewlet.pt')

    def isAvailable(self):
        return self.isTeacher or self.isManager


class ActionMenuViewletManager(OrderedViewletManager, PersonDistinguisher):
    """Viewlet manager for the action menu."""
    implements(IActionMenuContext, IActionMenuManager)

    def filterActions(self, actions):
        results = []
        for action in actions:
            if not self.isTeacher and IMessageBox.providedBy(self.context):
                if action['action'] == u'+/newMessage.html':
                    results.append(action)
            elif action['action'] == 'gradeCompetencies' and ISkillDriver.providedBy(self.context):
                if ISectionSkillDrivers.providedBy(self.context.__parent__):
                    results.append(action)
            else:
                results.append(action)
        return results

    def getSubItems(self, context):
        return list(subscribers((context, ), ISubMenuItem))

    def update(self):
        sub_items = self.getSubItems(self.context.__parent__)
        in_submenu_of_parent = self.context in sub_items
        displayed_in_submenu = IActionMenuManager.providedBy(self.__parent__)
        if (in_submenu_of_parent and not displayed_in_submenu):
            self.context = self.context.__parent__
        self.orderedViewletManagerUpdate()

    def orderedViewletManagerUpdate(self):
        OrderedViewletManager.update(self)

    def subItems(self):
        return self.getSubItems(self.context)
    
    def render(self):
        if self.isStudent and ICompetencies.providedBy(self.context):
            return ''
        return super(ActionMenuViewletManager, self).render()


class ReportMenuViewletManager(OrderedViewletManager, PersonDistinguisher):
    """Viewlet manager for the report menu."""
    implements(IActionMenuContext, IActionMenuManager)

    def filterActions(self, actions):
        results = []
        for action in actions:
            if not self.isTeacher and IMessageBox.providedBy(self.context):
                if action['action'] == u'+/newMessage.html':
                    results.append(action)
            elif action['action'] == 'gradeCompetencies' and ISkillDriver.providedBy(self.context):
                if ISectionSkillDrivers.providedBy(self.context.__parent__):
                    results.append(action)
            else:
                results.append(action)
        return results

    def getSubItems(self, context):
        return list(subscribers((context, ), ISubMenuItem))

    def update(self):
        sub_items = self.getSubItems(self.context.__parent__)
        in_submenu_of_parent = self.context in sub_items
        displayed_in_submenu = IActionMenuManager.providedBy(self.__parent__)
        if (in_submenu_of_parent and not displayed_in_submenu):
            self.context = self.context.__parent__
        self.orderedViewletManagerUpdate()

    def orderedViewletManagerUpdate(self):
        OrderedViewletManager.update(self)

    def subItems(self):
        return self.getSubItems(self.context)
    
    def render(self):
        if self.isStudent and ICompetencies.providedBy(self.context):
            return ''
        return super(ReportMenuViewletManager, self).render()


class ActionMenuViewlet(object):

    def getMenu(self):
        return getMenu(self.menu, self.context, self.request)


class HelpLink(SchoolToolHelpLink):
    """Help Link for Cando's New skin with its own page template."""

    implements(IContentProvider)

    template = ViewPageTemplateFile('helplink.pt')
