#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Skill Driver Tests

$Id$
"""
__docformat__='restructuredtext'

import unittest
import zope.component
from zope.testing import doctest
from zope.app.testing import setup
from zope.keyreference import testing
from schooltool.requirement import testing as requirementTesting
import schooltool.relationship.tests as relationship
from schooltool.course.interfaces import ICourse, ISection
from zope.intid.interfaces import IIntIds

from cando.virginia import competency
from cando.skilldriver.interfaces import ISkillDriver
from cando.virginia.interfaces import ICompetencyList

class IntIdsStub(object):

    def __init__(self):
        self.data = {}

    def getObject(self, id):
        return self.data.get(id)

    def register(self, ob):
        index = len(self.data)
        self.data[index] = ob

    def getId(self, ob):
        if self.queryId(ob) is not None:
            return self.queryId(ob)
        raise KeyError, repr(ob)

    def queryId(self, ob):
        for k,v in self.data.items():
            if v == ob:
                return k

def setUp(test):
    setup.placefulSetUp()
    setup.setUpAnnotations()
    relationship.setUpRelationships()
    zope.component.provideAdapter(competency.listAssignedCompetencies,
                                  (ICourse,),
                                  ICompetencyList)
    zope.component.provideAdapter(competency.listAssignedCompetencies,
                                  (ISkillDriver,),
                                  ICompetencyList)
    zope.component.provideAdapter(competency.listSectionCompetencies,
                                  (ISection,),
                                  ICompetencyList)
    zope.component.provideUtility(IntIdsStub(), IIntIds)
    requirementTesting.setUpEvaluation()
    requirementTesting.fixDecimal()
    zope.component.provideAdapter(testing.SimpleKeyReference)

def tearDown(test):
    setup.placefulTearDown()

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite(
            'README.txt',
            setUp=setUp, tearDown=tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS|
                        doctest.REPORT_ONLY_FIRST_FAILURE),
        ))

if __name__ == '__main__':
    unittest.main(default='test_suite')
