from schooltool.app.interfaces import ISchoolToolApplication
from cando.virginia.competency import getAppGlobalCompetencies
from cando.virginia import interfaces
from zope.interface import implements
from lxml import etree
from interfaces import ICompetencyXMLExporter


class CompetencyXMLExporter(object):
    """XML Export for competency list"""

    implements(ICompetencyXMLExporter)

    xmlns = "http://schooltool.org/schema"

    def __init__(self):
        self.root = None
        self.currentListID = 0
        self.xml = ''

    def exportToXML(self, competencies):
        self.xml = '<?xml version="1.0" ?>'
        if competencies is None:
            return False
        self._treeRecursionHelper(competencies, self.root)
        self.xml = etree.tostring(self.root, pretty_print=True)
        return True

    def _treeRecursionHelper(self, currentCompetencies, currentSubTree):

        if currentSubTree == None: #starting case
            self.root = etree.Element('{%s}complist' % self.xmlns,
                                      nsmap={'cando': self.xmlns})
            currentSubTree = self.root
        id = etree.SubElement(currentSubTree, "{%s}id" % self.xmlns)
        description = etree.SubElement(currentSubTree, "{%s}description" % self.xmlns)
        id.text = str(self.currentListID)
        self.currentListID += 1
        description.text = currentCompetencies.title
        for subCompObj in currentCompetencies.values():
            if interfaces.ICompetencyReference.providedBy(subCompObj):
                self._CompetencyXMLHelper(subCompObj, etree.SubElement(currentSubTree, "{%s}comp" % self.xmlns))
            else:
                self._treeRecursionHelper(subCompObj, etree.SubElement(currentSubTree, "{%s}complist" % self.xmlns))

    def _CompetencyXMLHelper(self, subComp, etreeElement):
        id = etree.SubElement(etreeElement, '{%s}id' % self.xmlns)
        title = etree.SubElement(etreeElement, '{%s}title' % self.xmlns)
        flag = etree.SubElement(etreeElement, '{%s}flag' % self.xmlns)
        globalId = etree.SubElement(etreeElement, '{%s}global-id' % self.xmlns)

        app = ISchoolToolApplication(None)
        idutil = app.getSiteManager().queryUtility(interfaces.IGlobalIdUtility)

        global_comps = getAppGlobalCompetencies(app)
        title.text = global_comps[subComp.globalId].title
        flag.text = subComp.optionalFlag
        id.text = subComp.id
        if subComp.globalId:
            globalId.text = subComp.globalId.replace(idutil.prefix, '', 1)
        else:
            globalId.text = ''

    def toPrettyXML(self):
        super = etree.tostring(self.root, pretty_print=True)
        return '<?xml version="1.0" ?>\n'+super
